leon_cald_config = {

    nav_hover_action: false,
    nav_mean: false,
    nav_mean_default_all: false,
    nav_pos: false,

    control: true,
    control_default_hide_cn: true,
    control_default_hide_example: false,

    pron_hover: false,
    pron_online: false,

    // if enabled, you can delete 2.mdd
    image_online: false,
    // if use online image, you can choose  fullsize image or thumb image
    image_online_fullsize: true,

    display_show_chinese: false,
    display_CN_click_display: false,
    display_CN_hover_display: false,

    display_collapse_unimportant: false,
    // hide first Thesaurus, More examples, Smart Vocabulary
    display_collapse_hide_first: true,
    // EXAMPLES of word
    display_example_hide: false,
    display_source: false,
}


cald_nav_mapping = {
    "combinations": "collocations"
}

// developer only, for dictionary software not support console.
cald_deubg = false

CALD_IMAGE_PREFIX = "https://dictionary.cambridge.org/images" + (leon_cald_config.image_online_fullsize ? "/full/" : "/thumb/");
CALD_AUDIO_PREFIX = "https://dictionary.cambridge.org/media/english/";

IS_CALD_LOADED = 'undefined' != typeof (IS_CALD_LOADED)
!IS_CALD_LOADED && document.addEventListener('DOMContentLoaded', function () {

    cald.addClickEvent()
    cald.elementDisplayAction()

    cald.addNavigation()
    leon_cald_config.nav_mean && cald.addMeanNavigation()

    cald.audioAndImageOnlineProcess()
    leon_cald_config.nav_pos && cald.addPos()
    leon_cald_config.control && cald.addControl()


    cald_deubg && vConsole()
    cald.hoverPron()
})

CALD_ROOT_CLASS = "leon-cald"
CALD_ROOT_SELECTOR = "." + CALD_ROOT_CLASS

CALD_NAV_CLASS = "nav-cald"
CALD_NAV_SELECTOR = "." + CALD_NAV_CLASS
CALD_NAV_MEAN_CLASS = "nav-mean-cald"
CALD_NAV_MEAN_SELECTOR = "." + CALD_NAV_MEAN_CLASS

CALD_NAV_POS_CLASS = "nav-pos-cald"
CALD_NAV_POS_SELECTOR = "." + CALD_NAV_POS_CLASS

CALD_ATTR_INDEX = "index";
// add global instance to window,avoid function pollution and conflict
cald = {
    root: function () {
        return document.querySelector(CALD_ROOT_SELECTOR);
    },

    addNavigation: function () {
        var root = cald.root()
        if (!root) {
            return;
        }

        if (cald.queryThisSelector(CALD_NAV_SELECTOR))
            return;

        let eles = cald.queryThisSelectorAll("#page-content >div.pr.di.superentry > div.di-body,.pr.dictionary,.pr.dataset");
        // 只有一个entry不添加
        if (eles.length < 2)
            return;
        let container = document.createElement('div');
        container.setAttribute("class", CALD_NAV_CLASS);
        for (let i = 0; i < eles.length; i++) {
            let sp = document.createElement('span');
            let pos = eles[i].getAttribute("data-id");
            sp.textContent = cald.navInfo(pos ? pos : "cald4");
            sp.setAttribute(CALD_ATTR_INDEX, i)
            if (i === 0) {
                sp.classList.add('active');
            }
            let navClickListener = function () {
                let index = +this.getAttribute(CALD_ATTR_INDEX);
                for (let j = 0; j < container.children.length; j++) {
                    container.children[j].classList.remove('active');
                }
                this.classList.add('active');
                cald.showHideEntry(index)
            };
            sp.addEventListener("click", navClickListener)
            leon_cald_config.nav_hover_action && sp.addEventListener("mouseover", navClickListener)
            container.appendChild(sp);
        }
        // 新增All
        let sp = document.createElement('span');
        sp.textContent = "All";
        container.appendChild(sp)
        let naviAllClickListener = function () {
            for (let j = 0; j < container.children.length; j++) {
                container.children[j].classList.remove('active');
            }
            this.classList.add('active');
            cald.showHideEntry(-1)
        };
        sp.addEventListener("click", naviAllClickListener)
        leon_cald_config.nav_hover_action && sp.addEventListener("mouseover", naviAllClickListener)
        cald.showHideEntry(0);
        root.insertBefore(container, root.childNodes[0]);
    },
    showHideEntry: function (index) {
        var root = cald.root();
        var eles = root.querySelectorAll(".pr.dictionary,.pr.dataset");
        return cald.showHideEntryOf(eles, index)
    },
    showHideEntryOf: function (eles, index) {
        for (var i = 0; i < eles.length; i++) {
            if (index === i || index < 0)
                showEle(eles[i]);
            else
                hideEle(eles[i]);
        }
    },
    navInfo: function (s) {
        return cald_nav_mapping[s] ? cald_nav_mapping[s] : s;
    },

    addMeanNavigation: function () {
        var root = cald.root()
        if (!root) {
            return;
        }
        // 已添加不添加
        if (root.querySelector(CALD_NAV_MEAN_SELECTOR))
            return;
        cald.elementsFn(".entry-body .entry-body__el", function (ele) {
            let posHeader = ele.querySelector(".pos-header");
            let posBody = ele.querySelector(".pos-body,.pv-body");

            if (posHeader && posBody) {
                if (cald_deubg) console.log(posHeader)
                let eles = posBody.children;
                // 只有一个entry不添加
                if (eles.length < 2)
                    return;
                var navContainer = document.createElement('div');
                navContainer.setAttribute("class", CALD_NAV_MEAN_CLASS);
                for (let i = 0; i < eles.length; i++) {
                    let sp = document.createElement('span');
                    let showEle = eles[i].querySelector("h3.dsense_h .dsense_gw span," +
                        ".phrase-head .phrase-title," +
                        ".def .x .x-h," +
                        ".xref h3 strong");
                    let pos = showEle ? showEle.textContent : i + "";
                    sp.textContent = pos ? pos : "unknown";
                    sp.setAttribute(CALD_ATTR_INDEX, i)
                    if (i === 0) {
                        sp.classList.add('active');
                    }
                    let navClickListener = function () {
                        var index = +this.getAttribute(CALD_ATTR_INDEX);
                        for (let j = 0; j < navContainer.children.length; j++) {
                            navContainer.children[j].classList.remove('active');
                        }
                        this.classList.add('active');
                        cald.showHideEntryOf(eles, index)
                    };
                    sp.addEventListener("click", navClickListener)
                    leon_cald_config.nav_hover_action && sp.addEventListener("mouseover", navClickListener)
                    navContainer.appendChild(sp);
                }

                if (navContainer.children.length > 1) {
                    // 新增All
                    let sp = document.createElement('span');
                    sp.textContent = "All";
                    navContainer.appendChild(sp)
                    let navAllClickListener = function () {
                        for (let j = 0; j < navContainer.children.length; j++) {
                            navContainer.children[j].classList.remove('active');
                        }
                        this.classList.add('active');
                        cald.showHideEntryOf(eles, -1)
                    };
                    sp.addEventListener("click", navAllClickListener)
                    leon_cald_config.nav_hover_action && sp.addEventListener("mouseover", navAllClickListener)
                    posHeader.appendChild(navContainer);
                    cald.showHideEntryOf(eles, 0)

                    leon_cald_config.nav_mean_default_all &&  navContainer.lastElementChild.click()
                }
            }
        })
    },

    addControl: function () {
        let root = cald.root();
        if (root.tagName !== "ARTICLE") {
            return
        }
        let hasExample = this.queryThisSelector(".ddef_h");
        let hasZh = this.queryThisSelector(".trans.zh");
        if (!hasExample && !hasZh) {
            return
        }
        let ctrl = document.createElement('div');
        ctrl.setAttribute("class", "cald_ctrl");
        if (hasZh) {
            let button = document.createElement('span');
            button.textContent = "ZH"
            button.addEventListener("click", function () {
                this.classList.toggle("enable")
                cald.elementsDisplay(".trans.zh", this.classList.contains("enable") ?
                    "inline-block" : "none")
            })

            if (leon_cald_config.display_show_chinese) {
                button.classList.add("enable")
                button.setAttribute("value", "ZH")
                ctrl.appendChild(button)
                leon_cald_config.control_default_hide_cn && button.click()
            }
        }
        if (hasExample) {
            let button2 = document.createElement('span');
            button2.textContent = "Example"
            button2.classList.add("enable")
            button2.addEventListener("click", function () {
                this.classList.toggle("enable")
                var collapsed = this.classList.contains("enable")
                cald.elementsFn(".ddef_h", function (ele) {
                    if (collapsed) {
                        ele.removeAttribute("expanded")
                    } else {
                        ele.setAttribute("expanded", "")
                    }
                })
            })
            if (leon_cald_config.control_default_hide_example) {
                button2.click()
            }
            ctrl.appendChild(button2)
        }

        if (ctrl.hasChildNodes()) {
            root.insertBefore(ctrl, root.childNodes[0])
        }
    },

    hoverPron: function () {
        leon_cald_config.pron_hover && cald.elementsFn(".i-volume-up", function (ele) {
            cald.elementEvent(ele, "mouseover", function (ev, el) {
                el.click()
            })
        })
    },

    elementDisplayAction: function () {
        leon_cald_config.display_example_hide && cald.elementsHide("#dataset-example>div:not(:first-child)");

        !leon_cald_config.display_source && cald.elementsHide("small,#dataset-example .dsource");

        leon_cald_config.display_show_chinese && cald.elementsDisplay(".trans.zh", "inline-block");

        leon_cald_config.display_collapse_hide_first && cald.elementsFn("amp-accordion:first-child section", function (ele) {
            ele.removeAttribute("expanded")
        })
        leon_cald_config.display_collapse_unimportant && cald.elementsFn(".grammar>h3:first-child," +
            ".phrasal_verbs>h3:first-child," +
            ".phrasal_verb>h3:first-child," +
            ".idioms>h3:first-child," +
            ".idiom>h3:first-child",
            function (ele) {
                ele.click()
            })
    },
    addClickEvent: function () {
        // current node sibling
        cald.elementsClick("#dataset-example>div:first-child," +
            ".grammar>h3:first-child," +
            ".phrasal_verbs>h3:first-child," +
            ".phrasal_verb>h3:first-child," +
            ".superentry>.di-head," +
            ".dsense>.dsense_h," +
            ".def-block .ddef_h," +
            ".idiom-block>.di-title," +
            ".pv-block>.di-title," +
            ".phrase-block>.phrase-head," +
            "div.idiom.hax>h3.bb," +
            ".cpexamps>.cpexamps-head," +
            ".idioms>h3:first-child",
            function (ele) {
                while (ele.nextElementSibling) {
                    var state = ele.nextElementSibling.style.display;
                    if (cald_deubg) console.log("state tg", state, !state, ele.nextElementSibling.tagName)
                    if (!leon_cald_config.display_source && ele.nextElementSibling.tagName === "SMALL") {
                        ele = ele.nextElementSibling
                        continue
                    }
                    if (state === "none") {
                        showEle(ele.nextElementSibling);
                        ele.classList.add('expanded');
                    } else {
                        hideEle(ele.nextElementSibling);
                        ele.classList.remove('expanded');
                    }
                    ele = ele.nextElementSibling
                }
            });

        // parent node sibling
        cald.elementsClick(".pos-header>.di-title", function (ele) {
            ele = ele.parentNode
            while (ele.nextElementSibling) {
                var state = ele.nextElementSibling.style.display;
                if (cald_deubg) console.log("state tg", state, !state)
                if (state === "none") {
                    showEle(ele.nextElementSibling);
                    ele.classList.add('expanded');
                } else {
                    hideEle(ele.nextElementSibling);
                    ele.classList.remove('expanded');
                }
                ele = ele.nextElementSibling
            }
        })

        // image full size
        cald.elementsClick("img", function (ele) {
            ele.classList.toggle("fullscreen")
        })

        // add  amp-accordion click
        cald.elementsClick("amp-accordion>section>header", function (ele) {
            var section = ele.parentNode;
            var expanded = section.hasAttribute("expanded");
            if (expanded) {
                section.removeAttribute("expanded")
            } else {
                section.setAttribute("expanded", "")
            }
        })

        // click hide show chinese
        let selector = ".examp .eg, .def,amp-accordion .eg";
        leon_cald_config.display_CN_click_display && cald.elementsClick(selector, function (ele, event) {
            event.stopPropagation()
            let zh = ele.querySelector(".trans.zh")
            if (zh) {
                zh.style.display = zh.style.display == "none" || zh.style.display == "" ? "inline-block" : "none";
            }
        })
        leon_cald_config.display_CN_hover_display && cald.elementsEvent(selector, "mouseover", function (ele, event) {
            let zh = ele.querySelector(".trans.zh")
            if (zh) {
                zh.style.display = zh.style.display == "none" || zh.style.display == "" ? "inline-block" : "none";
            }
        })
    },

    audioAndImageOnlineProcess: function () {
        leon_cald_config.image_online && cald.elementsFn("img", function (element) {
            if (cald_deubg) console.log("cald_online_image", element);
            element.src = cald.get_online_image_url(element.src);
            if (cald_deubg) console.log("cald_online_image after", element);
        })

        leon_cald_config.pron_online && cald.elementsFn("audio source,a.i-volume-up", function (element) {
            if (cald_deubg) console.log("cald_online_pron", element);
            if (element.src) {
                element.src = cald.get_online_pron_url(element.src);
            } else {
                element.href = cald.get_online_pron_url(element.href);
            }
        })
    },
    get_online_pron_url: function (src) {
        var ukIndex = src.indexOf("uk_pron/")
        var usIndex = src.indexOf("us_pron/")
        var name = src.substring(ukIndex > -1 ? ukIndex : usIndex);
        if (cald_deubg) console.log("get_online_pron_url", name)
        return CALD_AUDIO_PREFIX + name;
    },
    get_online_image_url: function (src) {
        var parts = src.split('/');
        var name = parts[parts.length - 1];
        if (cald_deubg) console.log("get_online_image_url", name)
        return CALD_IMAGE_PREFIX + name;
    },
    addPos: function () {
        var root = cald.root()
        if (!root) {
            return;
        }
        // 已添加不添加
        if (root.querySelector(CALD_NAV_POS_SELECTOR))
            return;
        var poses = []
        this.elementsFn(".pos-header .posgram .pos", function (ele) {
            if (poses.indexOf(ele.textContent) === -1) {
                poses.push(ele.textContent)
            }
        })
        if (poses.length > 1) {
            console.log("ssss")
            var posNavContainer = document.createElement('div');
            posNavContainer.setAttribute("class", CALD_NAV_POS_CLASS);
            for (let i = 0; i < poses.length; i++) {
                let sp = document.createElement('span');
                sp.textContent = poses[i] ? poses[i] : "unknown";
                sp.setAttribute(CALD_ATTR_INDEX, i)
                if (i === 0) {
                    sp.classList.add('active');
                }
                let navClickListener = function () {
                    var index = +this.getAttribute(CALD_ATTR_INDEX);
                    for (let j = 0; j < posNavContainer.children.length; j++) {
                        posNavContainer.children[j].classList.remove('active');
                    }
                    this.classList.add('active');
                    cald.showHidePosBody(poses[i])
                };
                sp.addEventListener("click", navClickListener)
                leon_cald_config.nav_hover_action && sp.addEventListener("mouseover", navClickListener)
                posNavContainer.appendChild(sp);
            }

            if (posNavContainer.children.length > 1) {
                // 新增All
                let sp = document.createElement('span');
                sp.textContent = "All";
                posNavContainer.appendChild(sp)
                let navAllClickListener = function () {
                    for (let j = 0; j < posNavContainer.children.length; j++) {
                        posNavContainer.children[j].classList.remove('active');
                    }
                    this.classList.add('active');
                    cald.showHidePosBody("All")
                };
                sp.addEventListener("click", navAllClickListener)
                leon_cald_config.nav_hover_action && sp.addEventListener("mouseover", navAllClickListener)

                root.insertBefore(posNavContainer, root.firstChild)
            }
        }
        console.log(poses)

    },
    showHidePosBody: function (pos) {
        this.elementsFn(".pos-header .posgram .pos", function (ele) {
            var entry = ele.closest(".entry-body__el")
            if (pos === ele.textContent || pos === "All") {
                showEle(entry)
            } else {
                hideEle(entry)
            }
        })
    },

    // common
    elementsFn: function (selector, fn) {
        selector = cald.thisSelector(selector)
        var eles = document.querySelectorAll(selector);
        for (var i = 0; i < eles.length; i++) {
            fn(eles[i]);
        }
    },

    elementsClick: function (selector, fn) {
        cald.elementsEvent(selector, "click", fn)
    },
    elementsEvent: function (selector, eventName, fn) {
        var events = eventName.split(",")
        for (let i = 0; i < events.length; i++) {
            cald.elementsFn(selector, function (ele) {
                ele.addEventListener(events[i], function (event) {
                    fn(ele, event);
                })
            })
        }
    },
    elementEvent: function (ele, eventName, fn) {
        var events = eventName.split(",")
        for (let i = 0; i < events.length; i++) {
            ele.addEventListener(events[i], function (event) {
                fn(event, ele);
            })
        }
    },

    thisSelector: function (selector) {
        return CALD_ROOT_SELECTOR + " " + selector.split(",").join("," + CALD_ROOT_SELECTOR + " ")
    },
    queryThisSelector: function (selector) {
        return document.querySelector(CALD_ROOT_SELECTOR + " " + selector.split(",").join("," + CALD_ROOT_SELECTOR + " "))
    },
    queryThisSelectorAll: function (selector) {
        return document.querySelectorAll(CALD_ROOT_SELECTOR + " " + selector.split(",").join("," + CALD_ROOT_SELECTOR + " "))
    },

    elementsDisplay: function elementsDisplay(selector, value) {
        cald.elementsFn(selector, function (ele) {
            ele.style.display = value;
        })
    },

    elementsHide: function (selector) {
        cald.elementsDisplay(selector, "none")
    },

    elementsShow: function elementsShow(selector) {
        cald.elementsDisplay(selector, "block")
    }
}

function get_elements() {
    var selector = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "input";
    var cond = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (el) {
        return el;
    };
    var eles = document.querySelectorAll(selector);
    var elements = [];
    for (var i = 0; i < eles.length; i++) {
        if (cond(eles[i])) {
            elements.push(eles[i]);
        }
    }
    return elements;
}

function hideEle(ele) {
    ele.style.display = 'none';
}

function showEle(ele) {
    ele.style.display = 'block';
}

function showEleInline(ele) {
    ele.style.display = 'inline-block';
}


function addScript(src, func) {
    let script = document.createElement('script');
    script.type = 'text/javascript';
    document.body.appendChild(script);
    if (src) {
        script.src = src;
    }
    script.onload = func;
}

function vConsole() {
    if ('undefined' == typeof (VConsole)) {
        addScript("https://cdn.bootcss.com/vConsole/3.15.1/vconsole.min.js", function () {
            var vConsole = new VConsole();
        })
    }
}